"""Tasks for creating debian or ubuntu images.

Please see docstring of each class for usage.

"""
from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *
from stacklet.task.stacktask import *
from stacklet.task.stackerror import *

class _XO_bootstrap(_XO_, StackTask):
    """Bootstrap a debian or ubuntu image.  Image will be chrootable upon completion
       of this task with network access to a package repository.


    Task Arguments:
    url -- the location to download packages from.
    arch -- the architecture (eg x86)
    release -- the release (eg 2008.0)
    proxy -- (optional) proxyhost[:port]

    """
    _TASK_NAME = 'bootstrap'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)

                env = {'http_proxy': self._proxy, 'PATH': '/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin'}

                cmd='mount -o loop ' + buildContext.get_global_asset_dir() + self._seed + ' ' + self._mountPoint
                self.exec_command(buildContext, cmd)

                cmd='cp -f /etc/resolv.conf ' + self._mountPoint + '/etc/'
                self.exec_command(buildContext, cmd)

                dirPath = self._mountPoint + '/tmp'
                cmd='mount --bind ' + buildContext.mountPoint + ' ' + dirPath
                self.exec_command(buildContext, cmd)

                bootstrapCmd = '/usr/sbin/chroot ' +  self._mountPoint + ' /usr/sbin/debootstrap --arch ' + self._arch + ' ' + self._version + ' /tmp ' + self._url
                self.exec_command(buildContext, bootstrapCmd, env=env)

                cmd='umount ' + dirPath
                self.exec_command(buildContext, cmd)

                cmd='umount ' + self._mountPoint
                self.exec_command(buildContext, cmd)

            except TaskError, taskErr:
                if os.path.ismount(self._mountPoint + '/tmp'):
                    cmd='fuser -k ' + self._mountPoint + '/tmp'
                    self.exec_command(buildContext, cmd, False)

                    cmd='umount ' + self._mountPoint + '/tmp'
                    self.exec_command(buildContext, cmd, False)

                if os.path.ismount(self._mountPoint):
                    cmd='fuser -k ' + self._mountPoint
                    self.exec_command(buildContext, cmd, False)

                    cmd='umount ' + self._mountPoint
                    self.exec_command(buildContext, cmd, False)

                taskErr.criticality = TaskError.ABORT_BUILD
                raise
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._seed = self.get_value('seed')
        self._mountPoint = self.get_value('mountPoint')
        self._url = self.get_value('url')
        self._arch = self.get_value('arch')
        self._version = self.get_value('version')
        self._proxy = self.get_value('proxy')

        if self.is_empty(self._seed):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'seed is required')

        if self.is_empty(self._mountPoint):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'mountPoint is required')

        if self.is_empty(self._url):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'url is required')

        if self.is_empty(self._arch):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'arch is required')

        if self.is_empty(self._version):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'version is required')


class _XO_package(_XO_, StackTask):
    """Install and/or remove package(s)

    Task Arguments:
    add -- space-delimited list of packages to install
    remove -- space-delimited list of packages to remove
    update -- space-delimited list of packages to update
    conf -- (optional) the location of yum.conf within the image
    proxy -- (optional) proxyhost[:port]

    """
    _TASK_NAME = 'package'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                cmds = list()

                env = {'http_proxy': self._proxy, 'PATH': '/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin'}

                if not self.is_empty(self._update):
                    self.log_info(buildContext, "update " + self._update)
                    if self._update == 'all':
                        cmd = 'apt-get --yes update; apt-get --yes upgrade'
                    cmds.append(cmd)

                if not self.is_empty(self._add):
                    self.log_info(buildContext, "add " + self._add)
                    cmd = 'apt-get --yes install ' + self._add
                    cmds.append(cmd)

                if not self.is_empty(self._remove):
                    self.log_info(buildContext, "remove " + self._remove)
                    cmd = 'apt-get --yes remove ' + self._remove
                    cmds.append(cmd)

                self.exec_command_chroot(buildContext, cmds, env=env)

            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._add = self.get_value('add')
        self._remove = self.get_value('remove')
        self._update = self.get_value('update')
        self._conf = self.get_value('conf')
        self._proxy = self.get_value('proxy')

        if self.is_empty(self._add) and self.is_empty(self._remove) and self.is_empty(self._update):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'add or remove or update is required')


class _XO_service(_XO_, StackTask):
    """Enable and/or disable service(s)

    Task Arguments:
    enable -- space-delimited list of services to enable
    disable -- space-delimited list of services to disable
    """
    _TASK_NAME = 'service'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                cmds = list()

                if not self.is_empty(self._enable):
                    self.log_info(buildContext, "enable " + self._enable)
                    for serv in self._enable.split():
                        cmds.append('update-rc.d -f ' + serv +  " defaults")

                if not self.is_empty(self._disable):
                    self.log_info(buildContext, "disable " + self._disable)
                    for serv in self._disable.split():
                        cmds.append('update-rc.d -f ' + serv +  " remove")

                self.exec_command_chroot(buildContext, cmds)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')


    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._enable = self.get_value('enable')
        self._disable = self.get_value('disable')

        if self.is_empty(self._enable) and self.is_empty(self._disable):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'enable or disable is required')

class _XO_cleanup(_XO_, StackTask):
    """Cleanup log files, yum cache, /tmp, etc...

    Task Arguments:
    zerofs -- if 'true' the empty space in the image will be zero'ed for better compressibility
    """
    _TASK_NAME = 'cleanup'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)

                cmds = list()
                cmds.append('apt-get clean')
                cmds.append('rm -f /var/log/dmesg*')
                cmds.append('rm -f /var/log/*.log')
                cmds.append('rm -f /var/log/mail.*')
                cmds.append('rm -f /var/log/messages')
                cmds.append('rm -f /var/log/syslog')
                cmds.append('rm -f /var/lib/dhcp/*')
                cmds.append('rm -f /var/lib/dhcp3/*')
                cmds.append('rm -f /var/lib/dhclient/*')
                cmds.append('rm -f /var/run/*.pid')
                cmds.append('rm -rf /var/spool/mail/*')
                cmds.append('rm -rf /var/tmp/*')
                cmds.append('rm -f /var/lock/*')
                cmds.append('rm -rf /root/.ssh')
                cmds.append('rm -f /root/.bash_history')
                cmds.append('rm -f /root/.nano_history')
                cmds.append('rm -f /etc/resolv.conf')
                cmds.append('rm -rf /bootstrap')
                cmds.append('rm -rf /tmp/*')

                self.exec_command_chroot(buildContext, cmds, checkReturnValue=False)

                if self._zerofs == 'true':
                    self.zero_empty_space(buildContext)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def onerr(self, buildContext, cmd, retval):
        pass

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)
        self._zerofs = self.get_value('zerofs')

gnosis.xml.objectify._XO_bootstrap = _XO_bootstrap
gnosis.xml.objectify._XO_stk_bootstrap = _XO_bootstrap
gnosis.xml.objectify._XO_package = _XO_package
gnosis.xml.objectify._XO_stk_package = _XO_package
gnosis.xml.objectify._XO_service = _XO_service
gnosis.xml.objectify._XO_stk_service = _XO_service
gnosis.xml.objectify._XO_cleanup = _XO_cleanup
gnosis.xml.objectify._XO_stk_cleanup = _XO_cleanup
