"""Tasks for creating various rpm/yum-based images.

Please see docstring of each class for usage.

"""
from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *
from stacklet.task.stacktask import *
from stacklet.task.stackerror import *

class _XO_bootstrap(_XO_, StackTask):
    """Bootstrap an rpm-based image.  Image will be chrootable upon completion
       of this task with network access to a package repository.

      Internally it downloads all rpms listed in <buildContext.mountPoint>/bootstrap/corePackages
      from argument url. Install the rpms to buildContext.mountPoint.
      Finally, copy the hosts's resolv.conf into the image.

      If the rpm list is setup properly, at this point it should be
      possible to chroot to buildContext.mountPoint and use yum and other
      tools to complete the image.

    Task Arguments:
    url -- the location to download packages from.  Ftp server is required if corePackages uses wildcards
    proxy -- (optional) proxyhost[:port]

    """
    _TASK_NAME = 'bootstrap'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)

                env = {'http_proxy': self._proxy}

                cmd = 'cp /etc/resolv.conf ' +  buildContext.mountPoint + '/etc/'
                self.exec_command(buildContext, cmd)

                cmd = 'wget -nv --base=' + self._url + ' -P ' + buildContext.mountPoint + '/bootstrap -i ' + self._corePackages
                self.exec_command(buildContext, cmd, env=env)

                cmd = 'rpm --root ' + buildContext.mountPoint + ' -i ' +  buildContext.mountPoint + '/bootstrap/*.rpm'
                self.exec_command(buildContext, cmd)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._url = self.get_value('url')
        self._corePackages = buildContext.mountPoint + '/bootstrap/corePackages'
        self._proxy = self.get_value('proxy')

        if self.is_empty(self._url):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'url is required')

        if not os.access(self._corePackages, os.F_OK):
            raise BuildError(self._TASK_NAME, TaskError.ABORT_TASK, 'Could not find ' + self._corePackages)

class _XO_packageGroup(_XO_, StackTask):
    """Install package group(s)

    Task Arguments:
    add -- space-delimited list of package groups to install
    conf -- (optional) the location of yum.conf within the image
    proxy -- (optional) proxyhost[:port]

    """
    _TASK_NAME = 'packageGroup'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                confArg = ''
                if not self.is_empty(self._conf):
                    confArg = ' -c ' + self._conf

                cmd = 'yum -y ' + confArg + ' groupinstall '  + self._add
                cmds = list()
                cmds.append(cmd)
                self.exec_command_chroot(buildContext, cmds, env=env)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._add = self.get_value('add')
        self._conf = self.get_value('conf')
        self._proxy = self.get_value('proxy')

        if self.is_empty(self._add):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'add is required')

class _XO_package(_XO_, StackTask):
    """Install and/or remove package(s)

    Task Arguments:
    add -- space-delimited list of packages to install
    remove -- space-delimited list of packages to remove
    update -- space-delimited list of packages to update
    conf -- (optional) the location of yum.conf within the image
    proxy -- (optional) proxyhost[:port]

    """
    _TASK_NAME = 'package'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                cmds = list()
                confArg = ''
                if not self.is_empty(self._conf):
                    confArg = ' -c ' + self._conf

                env = {'http_proxy': self._proxy}

                if not self.is_empty(self._update):
                    self.log_info(buildContext, "update " + self._update)
                    if self._update == 'all':
                        cmd = 'yum -y ' + confArg + ' update'
                    else:
                        cmd = 'yum -y ' + confArg + ' update ' + self._update
                    cmds.append(cmd)


                if not self.is_empty(self._add):
                    self.log_info(buildContext, "add " + self._add)
                    cmd = 'yum -y ' + confArg + ' install ' + self._add
                    cmds.append(cmd)


                if not self.is_empty(self._remove):
                    self.log_info(buildContext, "remove " + self._remove)
                    cmd = 'yum -y ' + confArg + ' remove ' + self._remove
                    cmds.append(cmd)

                self.exec_command_chroot(buildContext, cmds, env=env)

            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._add = self.get_value('add')
        self._remove = self.get_value('remove')
        self._update = self.get_value('update')
        self._conf = self.get_value('conf')
        self._proxy = self.get_value('proxy')

        if self.is_empty(self._add) and self.is_empty(self._remove) and self.is_empty(self._update):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'add or remove or update is required')

class _XO_service(_XO_, StackTask):
    """Enable and/or disable service(s)

    Task Arguments:
    enable -- space-delimited list of services to enable
    disable -- space-delimited list of services to disable
    """
    _TASK_NAME = 'service'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                cmds = list()

                if not self.is_empty(self._enable):
                    self.log_info(buildContext, "enable " + self._enable)
                    for serv in self._enable.split():
                        cmds.append('chkconfig ' + serv + ' on')

                if not self.is_empty(self._disable):
                    self.log_info(buildContext, "disable " + self._disable)
                    for serv in self._disable.split():
                        cmds.append('chkconfig ' + serv + ' off')

                self.exec_command_chroot(buildContext, cmds)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')


    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._enable = self.get_value('enable')
        self._disable = self.get_value('disable')

        if self.is_empty(self._enable) and self.is_empty(self._disable):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'enable or disable is required')

class _XO_cleanup(_XO_, StackTask):
    """Cleanup log files, yum cache, /tmp, etc...

    Task Arguments:
    zerofs -- if 'true' the empty space in the image will be zero'ed for better compressibility
    """
    _TASK_NAME = 'cleanup'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)

                cmds = list()
                cmds.append('yum clean all')
                cmds.append('rm -f /var/log/*')
                cmds.append('rm -f /var/log/audit/*')
                cmds.append('rm -f /var/lib/dhcp/*')
                cmds.append('rm -f /var/lib/dhclient/*')
                cmds.append('rm -f /var/run/*.pid')
                cmds.append('rm -rf /var/spool/mail/*')
                cmds.append('rm -rf /var/tmp/*')
                cmds.append('rm -f /var/lock/*')
                cmds.append('rm -rf /root/.ssh')
                cmds.append('rm -f /root/.bash_history')
                cmds.append('rm -f /root/.nano_history')
                cmds.append('rm -f /etc/resolv.conf')
                cmds.append('rm -rf /bootstrap')
                cmds.append('rm -rf /tmp/*')

                self.exec_command_chroot(buildContext, cmds, checkReturnValue=False)

                if self._zerofs == 'true':
                    self.zero_empty_space(buildContext)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def onerr(self, buildContext, cmd, retval):
        pass

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)
        self._zerofs = self.get_value('zerofs')

gnosis.xml.objectify._XO_bootstrap = _XO_bootstrap
gnosis.xml.objectify._XO_stk_bootstrap = _XO_bootstrap
gnosis.xml.objectify._XO_packageGroup = _XO_packageGroup
gnosis.xml.objectify._XO_stk_packageGroup = _XO_packageGroup
gnosis.xml.objectify._XO_package = _XO_package
gnosis.xml.objectify._XO_stk_package = _XO_package
gnosis.xml.objectify._XO_service = _XO_service
gnosis.xml.objectify._XO_stk_service = _XO_service
gnosis.xml.objectify._XO_cleanup = _XO_cleanup
gnosis.xml.objectify._XO_stk_cleanup = _XO_cleanup
