"""Tasks for creating slackware images.

Please see docstring of each class for usage.

"""
from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *
from stacklet.task.stacktask import *
from stacklet.task.stackerror import *

class _XO_bootstrap(_XO_, StackTask):
    """Bootstrap a slackware image.  Image will be chrootable upon completion
       of this task with network access to a package repository.

      Internally it downloads all packages listed in <buildContext.mountPoint>/bootstrap/corePackages
      from argument url. Install the packages to buildContext.mountPoint.
      Finally, copy the hosts's resolv.conf into the image.

    Task Arguments:
    url -- the location to download packages from.  Ftp server is required if corePackages uses wildcards
    proxy -- (optional) proxyhost[:port]

    """
    _TASK_NAME = 'bootstrap'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                env = {'http_proxy': self._proxy}

                slackURL = self._url + 'slackware/'
                cmd = 'wget -nv --base=' + slackURL + ' -P ' + buildContext.mountPoint + '/bootstrap -i ' + self._corePackages
                self.exec_command(buildContext, cmd, env=env)

                cmd='tar -xzpf ' + buildContext.mountPoint + '/bootstrap/aaa_base-*.tgz' + ' -C ' + buildContext.mountPoint
                self.exec_command(buildContext, cmd)

                cmd='cp ' + buildContext.inputDocAbsoluteFilePath + '/assets/busybox ' + buildContext.mountPoint + '/bin/'
                self.exec_command(buildContext, cmd)

                cmd='cp /etc/resolv.conf ' + buildContext.mountPoint + '/etc/'
                self.exec_command(buildContext, cmd)

                commands = list()
                commands.append('. /install/doinst.sh')
                self.exec_command_chroot_busybox(buildContext, commands)

                self._install_bootstrap_package('etc', buildContext)
                self._install_bootstrap_package('devs', buildContext)

                cmd='tar -xzpf ' + buildContext.mountPoint + '/bootstrap/glibc-2*.tgz' + ' -C ' + buildContext.mountPoint
                self.exec_command(buildContext, cmd)

                commands = list()
                commands.append('ln -s /bin/busybox /bin/cp')
                commands.append('. /install/doinst.sh')
                commands.append('rm /bin/cp')
                self.exec_command_chroot_busybox(buildContext, commands)

                self._install_bootstrap_package('coreutils', buildContext)
                self._install_bootstrap_package('libtermcap', buildContext)
                self._install_bootstrap_package('bash', buildContext)
                self._install_bootstrap_package('tar', buildContext)
                self._install_bootstrap_package('gzip', buildContext)
                self._install_bootstrap_package('grep', buildContext)
                self._install_bootstrap_package('bin', buildContext)
                self._install_bootstrap_package('pkgtools', buildContext)
                self._install_bootstrap_package('which', buildContext)
                self._install_bootstrap_package('sed', buildContext)
                self._install_bootstrap_package('aaa_elflibs', buildContext)
                self._install_bootstrap_package('gawk', buildContext)
                self._install_bootstrap_package('wget', buildContext)
                self._install_bootstrap_package('openssl-solibs', buildContext)
                self._install_bootstrap_package('bzip2', buildContext)

                cmd = 'wget -c -P ' + buildContext.mountPoint + '/bootstrap http://slackpkg.org/stable/slackpkg-2.70.4-noarch-1.tgz'
                self.exec_command(buildContext, cmd, env=env)
                self._install_bootstrap_package('slackpkg', buildContext)

                commands = list()
                commands.append("installpkg /bootstrap/*.tgz")
                commands.append("sed -i.save -e 's/DIALOG=on/DIALOG=off/' /etc/slackpkg/slackpkg.conf")
                commands.append("sed -i -e 's/BATCH=off/BATCH=on/' /etc/slackpkg/slackpkg.conf")
                commands.append("sed -i -e 's/DEFAULT_ANSWER=n/DEFAULT_ANSWER=y/' /etc/slackpkg/slackpkg.conf")
                commands.append("sed -i -e 's/CHECKGPG=on/CHECKGPG=off/' /etc/slackpkg/slackpkg.conf")
                commands.append('echo ' + self._url + ' >> /etc/slackpkg/mirrors')
                commands.append('slackpkg update')
                self.exec_command_chroot(buildContext, commands, env=env)

            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._url = self.get_value('url')
        self._corePackages = buildContext.mountPoint + '/bootstrap/corePackages'
        self._proxy = self.get_value('proxy')

        if self.is_empty(self._url):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'url is required')

        if not os.access(self._corePackages, os.F_OK):
            raise BuildError(self._TASK_NAME, TaskError.ABORT_TASK, 'Could not find ' + self._corePackages)


    def _install_bootstrap_package(self, package, buildContext):

      cmd='tar -xzpf ' + buildContext.mountPoint + '/bootstrap/' + package + '*.tgz' + ' -C ' + buildContext.mountPoint
      self.exec_command(buildContext, cmd)

      commands = list()
      commands.append('. /install/doinst.sh')
      self.exec_command_chroot_busybox(buildContext, commands)


class _XO_package(_XO_, StackTask):
    """Install and/or remove package(s)

    Task Arguments:
    add -- space-delimited list of packages to install
    remove -- space-delimited list of packages to remove
    update -- space-delimited list of packages to update
    proxy -- (optional) proxyhost[:port]
    """
    _TASK_NAME = 'package'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                env = {'http_proxy': self._proxy}

                cmds = list()

                if not self.is_empty(self._update):
                    self.log_info(buildContext, "update " + self._update)
                    if self._update == 'all':
                        cmd = 'slackpkg update'
                        cmds.append(cmd)
                        cmd = 'slackpkg upgrade-all'
                        cmds.append(cmd)

                if not self.is_empty(self._add):
                    self.log_info(buildContext, "add " + self._add)
                    cmd = 'slackpkg install ' + self._add
                    cmds.append(cmd)

                if not self.is_empty(self._remove):
                    self.log_info(buildContext, "remove " + self._remove)
                    cmd = 'removepkg ' + self._remove
                    cmds.append(cmd)

                self.exec_command_chroot(buildContext, cmds, env=env)

            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._add = self.get_value('add')
        self._remove = self.get_value('remove')
        self._update = self.get_value('update')
        self._proxy = self.get_value('proxy')

        if self.is_empty(self._add) and self.is_empty(self._remove) and self.is_empty(self._update):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'add or remove or update is required')

class _XO_service(_XO_, StackTask):
    """Enable and/or disable service(s)

    Task Arguments:
    enable -- space-delimited list of services to enable
    disable -- space-delimited list of services to disable
    """
    _TASK_NAME = 'service'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                cmds = list()

                if not self.is_empty(self._enable):
                    self.log_info(buildContext, "enable " + self._enable)
                    for serv in self._enable.split():
                        cmds.append('chmod +x /etc/rc.d/rc.' + serv)

                if not self.is_empty(self._disable):
                    self.log_info(buildContext, "disable " + self._disable)
                    for serv in self._disable.split():
                        cmds.append('chmod -x /etc/rc.d/rc.' + serv)

                self.exec_command_chroot(buildContext, cmds)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')


    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._enable = self.get_value('enable')
        self._disable = self.get_value('disable')

        if self.is_empty(self._enable) and self.is_empty(self._disable):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'enable or disable is required')

class _XO_cleanup(_XO_, StackTask):
    """Cleanup log files, /tmp, etc...

    Task Arguments:
    zerofs -- if 'true' the empty space in the image will be zero'ed for better compressibility
    """
    _TASK_NAME = 'cleanup'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)

                cmds = list()
                cmds.append('rm -f /var/log/*')
                cmds.append('rm -f /var/log/removed_packages/*')
                cmds.append('rm -f /var/log/removed_scripts/*')
                cmds.append('rm -f /var/state/dhcp/*')
                cmds.append('rm -f /var/run/*.pid')
                cmds.append('rm -rf /var/tmp/*')
                cmds.append('rm -f /var/lock/*')
                cmds.append('rm -rf /root/.ssh')
                cmds.append('rm -rf /root/.bash_history')
                cmds.append('rm -f /root/.nano_history')
                cmds.append('rm -f /etc/resolv.conf')
                cmds.append('rm -rf /glibc-tmp-*')
                cmds.append('rm -rf /bootstrap')
                cmds.append('rm -rf /bin/busybox')
                cmds.append('rm -rf /tmp/*')
                self.exec_command_chroot(buildContext, cmds, checkReturnValue=False)

                if self._zerofs == 'true':
                    self.zero_empty_space(buildContext)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def onerr(self, buildContext, cmd, retval):
        pass

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)
        self._zerofs = self.get_value('zerofs')

gnosis.xml.objectify._XO_bootstrap = _XO_bootstrap
gnosis.xml.objectify._XO_stk_bootstrap = _XO_bootstrap
gnosis.xml.objectify._XO_cleanup = _XO_cleanup
gnosis.xml.objectify._XO_stk_cleanup = _XO_cleanup
gnosis.xml.objectify._XO_package = _XO_package
gnosis.xml.objectify._XO_stk_package = _XO_package
gnosis.xml.objectify._XO_service = _XO_service
gnosis.xml.objectify._XO_stk_service = _XO_service

