"""Tasks for creating files.

Please see docstring of each class for usage.

"""
import os
import re
import urllib

from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *

import stacklet.util.stringutil
from stacklet.task.stacktask import *
from stacklet.task.stackerror import *

class _XO_content(_XO_, StackTask):
    """Provides content to the enclosing file task.  Content is stored in self.content
       for use by its parent file task.

    There are three basic ways to use this task:

    The first is simply to embed CDATA inside of the content tag.
    This data will then be loaded into self.content; note that only CDATA text
    will be used.  Any xml children inside of the content tag will be ignored.
    Explicitly use <![CDATA[...]]> otherwise the xml parser will normalize whitespace

    The second is to use an <xi:include href="..." parse="text"/> to provide content.
    This is functionally the same as the first except that the CDATA is included
    via an xinclude.

    The third way is to specify loc and name arguments which the task will use to
    construct a url.  The contents of the url will be loaded to self.content.
    Currently http, ftp and file urls are supported.  Relative file paths
    are also supported (i.e. loc will be relative to the directory containing
    the xml file being processed).  Name is *required* if you wish
    to use this method of retrieving content.  Loc will default to '.' if it
    is absent (meaning the directory containing the xml file being processed)

      Examples:
        1. Get content from /tmp/myfile (in the builder's filesystem!)
          <loc>file:///tmp</loc>
          <name>myfile</name>
        2. Get content from /tmp/myfile (in the builder's filesystem!)
          <loc>/tmp</loc>
          <name>myfile</name>
        3. Get content from /tmp/myfile (in the image's filesystem!)
          <loc>_image.mountPoint_/tmp</loc>
          <name>myfile</name>
        4. Get content from google
          <loc>http://www.google.com</loc>
          <name>index.html</name>
        5. Get content from a file that is located in assets/files directory
           relative to the xml file being processed
          <loc>assets/files</loc>
          <name>myfile</name>

    Task Arguments:
    loc -- (optional) the directory or url containing the file to use as the basis for the content
    name -- the name of the file containing the content.  This is required when using url based retrievals
            Should only contain the name of the file, path info should be specified in loc.
    filters -- (optional) a space separated list of filters to perform on the content
               substitute_vars: replace all variables embedded in the content
               flush_left: remove leading spaces from *all* lines but keep relative indentation
               strip: remove leading and trailing space from the content but add a single newline at end

               The latter two filters used together are esp. helpful when CDATA
               is embedded directly in the content tag.

    """
    _TASK_NAME = 'content'

    def __init__(self):
        self.content = ''

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)

                if not self.is_empty(self._url):
                    f = urllib.urlopen(self._url)
                    self.content = f.read()
                    f.close()
                else:
                    self.content = dumps(self)

                if not self.is_empty(self._filters):
                    for filt in self._filters.split():
                        if filt == 'substitute_props':
                            self.content = stacklet.util.stringutil.substitute_vars(self._buildContext.get_all_properties(), self.content)
                        elif filt == 'flush_left':
                            self.content = stacklet.util.stringutil.flush_left(self.content)
                        elif filt == 'strip':
                            self.content = self.content.strip() + '\n'
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._loc = self.get_value('loc')
        self._name = self.get_value('name')
        self._filters = self.get_value('filters')
        self._url = ''

        if not self.is_empty(self._name):
            if self.is_empty(self._loc):
                self._loc = './'
            if not self._loc.endswith(os.sep):
                self._loc = self._loc + os.sep
            if not self._loc.startswith('.') and not self._loc.startswith(os.sep)\
                                                     and not self._loc.startswith('file://')\
                                                     and not self._loc.startswith('http://')\
                                                     and not self._loc.startswith('ftp://'):
               self._loc = './' + self._loc
            if self._loc.startswith('..'):
               self._loc = 'file://' + buildContext.inputDocAbsoluteFilePath + os.sep + self._loc
            elif self._loc.startswith('.'):
               self._loc = 'file://' + buildContext.inputDocAbsoluteFilePath + self._loc[1:]
            elif self._loc.startswith('/'):
                self._loc = 'file://' + self._loc
            self._url = self._loc + self._name

class _XO_file(_XO_, StackTask):
    """Create a file inside of the image under construction.

    This task should be used in conjunction with the content task.  A file
    tag without a child content tag will create an empty file.  A file tag with
    one or more content tags will have its contents derived from its children.

    Task Arguments:
    loc -- the parent directory of the file (this directory will be created if necessary)
           Note: the build's mount point (altroot) will be automatically prepended to
                 loc, i.e. the loc is relative to the altroot and must start with '/'
    name -- the name of the file
    exists -- (optional) what to do if the file already exists (archive|leave|replace)
    mode -- (optional) set the file's mode
    owner -- (optional) set the file's owner and group
    """
    _TASK_NAME = 'file'
    _EXISTS_OPTIONS = ['archive', 'leave', 'replace']

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                parentDir = buildContext.mountPoint + self._loc
                fullFilePath = parentDir + os.sep + self._name
                if os.access(fullFilePath, os.F_OK):
                    if self._exists == 'replace':
                        os.remove(fullFilePath)
                    elif self._exists == 'archive':
                        os.rename(fullFilePath, fullFilePath + '.save')
                    elif self._exists == 'leave':
                        return
                else:
                    if os.access(parentDir, os.F_OK):
                        if not os.path.isdir(parentDir):
                            raise BuildError(self._TASK_NAME, TaskError.ABORT_TASK, parentDir + ' is not a directory.')
                    else:
                        cmd='mkdir -p ' + parentDir
                        self.exec_command(buildContext, cmd)

                content = ''
                for child in children(self):
                    if isinstance(child, _XO_content):
                        child.process(buildContext)
                        content = content + child.content

                f = open(fullFilePath, 'w')
                f.write(content)
                f.close()

                if not self.is_empty(self._mode):
                    cmd="chmod " + self._mode + ' ' + fullFilePath
                    self.exec_command(buildContext, cmd)

                if not self.is_empty(self._owner):
                    cmds = list()
                    cmd="chown " + self._owner + ' ' + self._loc + os.sep + self._name
                    cmds.append(cmd)
                    self.exec_command_chroot(buildContext, cmds) #run chown inside a chroot!

            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._loc = self.get_value('loc')
        self._name = self.get_value('name')
        self._exists = self.get_value('exists')
        self._mode = self.get_value('mode')
        self._owner = self.get_value('owner')

        if self.is_empty(self._loc):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'loc is required')
        elif not self._loc.startswith(os.sep):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'loc must start with ' + os.sep)

        if self.is_empty(self._name):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'name is required')

        if not self.is_empty(self._exists):
            if not (self._exists in self._EXISTS_OPTIONS):
                raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'exists must be one of ' + str(self._EXISTS_OPTIONS) + ' encountered ' + self._exists)
        else:
            self._exists = 'archive'


gnosis.xml.objectify._XO_content = _XO_content
gnosis.xml.objectify._XO_stk_content = _XO_content
gnosis.xml.objectify._XO_file = _XO_file
gnosis.xml.objectify._XO_stk_file = _XO_file
