"""Tasks for creating images.

Please see docstring of each class for usage.

"""
import os
import re
import urllib

from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *

from stacklet.task.stacktask import *
from stacklet.task.stackerror import *



class _XO_image(_XO_, StackTask):
    """The top-level container for other tasks.  Creates (or mounts) an image.

    Task Arguments:
    imgFile -- the complete path and name of the image file to create/mount
    mountPoint -- where to mount the imgFile during execution.  Image will be unmounted on exit.
    size -- size in MB of the imgFile, only required if creating a new image
    exists -- (optional, default='archive') what to do if the file already exists (archive|mount|replace)
    fsType -- (optional, default='ext3') the filesystem type of imgFile, ignored if mounting an existing image
    """
    _TASK_NAME = 'image'
    _EXISTS_OPTIONS = ['archive', 'mount', 'replace']
    _FSTYPE_OPTIONS = ['ext2', 'ext3']

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self._mountExecuted = False
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)

                createNew = False
                if os.access(self._imgFile, os.F_OK):
                    if self._exists == 'replace':
                        os.remove(self._imgFile)
                        createNew = True
                    elif self._exists == 'archive':
                        os.rename(self._imgFile, self._imgFile + '.save')
                        createNew = True
                else:
                    createNew = True

                if createNew:
                    cmd='dd if=/dev/zero of=' + self._imgFile + ' bs=1M count=1 seek=' + self._size
                    self.exec_command(buildContext, cmd)

                    if self._fsType == 'ext3':
                        cmd='/sbin/mke2fs -F -j ' + self._imgFile
                    else:
                        cmd='/sbin/mke2fs -F ' + self._imgFile
                    self.exec_command(buildContext, cmd)

                cmd='mount -o loop ' + self._imgFile + ' ' + buildContext.mountPoint
                self.exec_command(buildContext, cmd)
                self._mountExecuted = True

                cmd='mkdir -p ' + buildContext.mountPoint + '/proc'
                self.exec_command(buildContext, cmd)

                cmd='mount -t proc none ' + buildContext.mountPoint + '/proc'
                self.exec_command(buildContext, cmd)

                for child in children(self):
                    if isinstance(child, StackTask):
                        child.process(buildContext)

            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            try:
                if self._mountExecuted and os.path.ismount(self._mountPoint):
                    cmd='fuser ' + self._mountPoint + ' && sleep 10'
                    self.exec_command(buildContext, cmd, False)
                    cmd='fuser -k ' + self._mountPoint
                    self.exec_command(buildContext, cmd, False)
                    if os.path.ismount(self._mountPoint + '/proc'):
                        cmd='umount ' + self._mountPoint + '/proc'
                        self.exec_command(buildContext, cmd, False)
                    cmd='umount ' + self._mountPoint
                    self.exec_command(buildContext, cmd, False)
            finally:
                if self._mountExecuted and os.path.ismount(self._mountPoint):
                    self.log_error(buildContext, 'Could not unmount ' + self._mountPoint)

            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self._exists = self.get_value('exists')
        self._imgFile = self.get_value('imgFile')
        self._size = self.get_value('size')
        self._fsType = self.get_value('fsType')
        self._mountPoint = self.get_value('mountPoint')

        if self.is_empty(self._imgFile):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'imgFile is required')

        if self.is_empty(self._mountPoint):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'mountPoint is required')
        else:
            if not os.access(self._mountPoint, os.F_OK) or not os.path.isdir(self._mountPoint):
                raise BuildError(self._TASK_NAME, TaskError.ABORT_TASK, 'mountPoint ' + self._mountPoint + ' is not a directory.')
            if os.path.ismount(self._mountPoint):
                raise BuildError(self._TASK_NAME, TaskError.ABORT_TASK, 'already mounted:' + self._mountPoint)
            buildContext.mountPoint = self._mountPoint

        if not self.is_empty(self._exists):
            if not (self._exists in self._EXISTS_OPTIONS):
                raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'exists must be one of ' + str(self._EXISTS_OPTIONS) + ' encountered ' + self._exists)
        else:
            self._exists = 'archive'

        if self.is_empty(self._size):
            if not self._exists == 'mount':
                raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'size is required')
        else:
            if not self._size.isdigit():
                raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'size must be numeric')

        if not self.is_empty(self._fsType):
            if not (self._fsType in self._FSTYPE_OPTIONS):
                raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'fsType must be one of ' + str(self._FSTYPE_OPTIONS) + ' encountered ' + self._exists)
        else:
            self._fsType = 'ext3'

gnosis.xml.objectify._XO_image = _XO_image
gnosis.xml.objectify._XO_stk_image = _XO_image

