"""Base class for all Tasks

Provides utility methods for use by subclasses.

"""
import os
import stat
import subprocess
import re
import types
import select

import stacklet.util.stringutil
from stacklet.task.stackerror import *

class StackTask:
    def __init__(self):
        pass

    def exec_command(self, buildContext, cmd, checkReturnValue=True, env=os.environ):
        """Execute a shell command

        If checkReturnValue=True and the command returns a non-zero value then
        the executing task's onerr method will be called.  If the task does
        not have an onerr method then a ExecError will be thrown.  Passing
        checkReturnValue=False will bypass this behavior.

        arguments:
        buildContext -- the build context
        cmd -- a string containing the command to execute
        checkReturnValue -- boolean controlling whether to interrogate the command's return value
        env -- Defines the environment variables for the new process.  Defaults to current env if not passed
        """
        self.log_info(buildContext, 'exec_command ' + cmd)
        proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)
        while proc.poll() is None:
            self._print_pipes(buildContext, proc)
        self._print_pipes(buildContext, proc)
        retval = proc.returncode
        if retval != 0 and checkReturnValue:
            if hasattr(self, 'onerr'):
                self.onerr(buildContext, cmd, retval)
            else:
                execErr = ExecError(self._TASK_NAME, TaskError.ABORT_TASK, 'shell return code:' + str(retval))
                raise execErr

    def _print_pipes(self, buildContext, proc):
        """Print the output from exec_command using the logging facility;
        differentiate stdout and stderr using prefixes.  """

        keepGoing = True
        while keepGoing:
            keepGoing = False
            read_set = []
            read_set.append(proc.stdout)
            read_set.append(proc.stderr)
            rlist, wlist, xlist = select.select(read_set, [], [])

            if proc.stdout in rlist:
                data = proc.stdout.readline()
                if not data == '':
                    keepGoing = True
                    self.log_info(buildContext, '\tstdout:' + data.strip())
            if proc.stderr in rlist:
                data = proc.stderr.readline()
                if not data == '':
                    keepGoing = True
                    self.log_info(buildContext, '\tstderr:' + data.strip())

    def exec_command_chroot(self, buildContext, cmds, checkReturnValue=True, env=None):
        """Execute a shell command or commands in the image under construction,
        using chroot with the image's mount point as the root.

        Notes:
        1. the command(s) are run using the bash shell in the chroot
        2. environment variables are *not* inherited in the chroot
        3. each command is stored as a "script" in the chroot's /tmp dir

        arguments:
        buildContext -- the build context
        cmds -- a list or string containing the command(s) to execute
        checkReturnValue -- boolean controlling whether to interrogate the command's return value
        env -- Defines the environment variables for the new process.  Defaults to None if not passed
        """
        if isinstance(cmds, types.StringTypes):
            cmds = [cmds]

        for cmd1 in cmds:
            cmdFile =  '/tmp/commands.' + str(buildContext.counter) + '.sh'
            buildContext.counter = buildContext.counter + 1
            f = open(buildContext.mountPoint + cmdFile, 'w')
            f.write(cmd1 +  "\n")
            f.close()
            os.chmod(buildContext.mountPoint + cmdFile, stat.S_IEXEC)

            cmd='/usr/sbin/chroot ' + buildContext.mountPoint + ' bash -l ' + cmdFile
            if env is None:
                cmd='env - ' + cmd

            self.log_info(buildContext, 'exec_command_chroot ' + cmd1)
            self.exec_command(buildContext, cmd, checkReturnValue, env)


    def exec_command_chroot_busybox(self, buildContext, cmds, checkReturnValue=True):
        """Execute a shell command or commands in the image under construction,
        using chroot with the image's mount point as the root.

        Notes:
        1. the command(s) are run using busybox in the chroot (must already exist)
        2. environment variables are *not* inherited in the chroot

        arguments:
        buildContext -- the build context
        cmds -- a list or string containing the command(s) to execute
        checkReturnValue -- boolean controlling whether to interrogate the command's return value
        """
        if isinstance(cmds, types.StringTypes):
            cmds = [cmds]

        for cmd1 in cmds:
            cmd='env - /usr/sbin/chroot ' + buildContext.mountPoint + ' /bin/busybox ash -c "' + cmd1 + '"'
            self.log_info(buildContext, 'exec_command_chroot_busybox ' + cmd1)
            self.exec_command(buildContext, cmd, checkReturnValue)

    def zero_empty_space(self, buildContext):
        """Zero out the empty space in the image under construction
        for better compressibility.

        arguments:
        buildContext -- the build context
        """
        try:
            outFileName = buildContext.mountPoint + "/tmp/zeros"
            s = os.statvfs(buildContext.mountPoint)
            availableMB = s.f_bsize * s.f_bfree / 1000000

            zeros = open("/dev/zero", 'r')
            zstr = zeros.read(1024 * 1024)
            outf = open(outFileName, 'w')

            for n in range(1, availableMB):
                outf.write(zstr)
        except IOError:
            pass # "IOError: [Errno 28] No space left on device" will be thrown in this method, ignore it
        except:
            self.log_error(buildContext, 'Error in zero_empty_space') # log any other error but do not re-raise

        if os.access(outFileName, os.F_OK):
            os.remove(outFileName)

    def get_value(self, name, substituteProperties=True):
        """Retrieves a value the task will use to control execution.

        Generally these values can be considered the arguments to the task and are encoded
        as attributes or child nodes of the task.  This method treats
        attributes and child nodes interchangably.  In other words these are equivalent:

        1. <mytask taskarg='blue'/>
        2. <mytask><taskarg>blue</taskarg></mytask>
        In either case get_value('taskarg') returns 'blue'

        If nothing corresponding to name can be found, '' is returned
        get_value('nonarg') returns ''

        If name is None, then the raw PCDATA contained within the task will be returned:
        1. <mytask>Here is some raw data</mytask>
        get_value(None) returns 'Here is some raw data'

        If multiple values correspond to the name then a BuildError is thrown, eg:
        1. <mytask><taskarg>blue</taskarg><taskarg>red</taskarg></mytask>
        get_value('taskarg') --> throws exception

        Finally, substituteProperties controls whether property replacement
        occurs.  If True, then any occurence of variables in the value will be replaced
        based on the properties stored in the build context.  Eg: assuming that
        the build context has a property _color_ = 'green'
        <mytask><taskarg>_color_</taskarg></mytask>
        get_value('taskarg', substituteProperties=True) --> 'green'
        get_value('taskarg', substituteProperties=False) --> '_color_'

        arguments:
        name -- the name of the argument whose value is to be retrieved
        substituteProperties -- boolean controlling whether property substitution occurs on the value
        """
        val = ''
        if name is None:
            if hasattr(self, 'PCDATA'):
                val = self.PCDATA
            else:
                return ''
        elif hasattr(self, name):
            node = getattr(self, name)
            if isinstance(node, types.ListType):
                raise BuildError(self._TASK_NAME, TaskError.ABORT_TASK, 'expecting single value but encountered list for argument: ' + name)
            elif hasattr(node, 'PCDATA'):
                val = node.PCDATA
            else:
                val = node

        if substituteProperties:
            val = stacklet.util.stringutil.substitute_vars(self._buildContext.get_all_properties(), val)
        return val

        return None

    def is_empty(self, val):
        """Returns True if val is None, an empty string or all whitespace; False otherwise """
        if val is None or val.strip() == '':
            return True
        return False;

    def log_error(self, buildContext, taskErr):
        """Log an exception using the logging facility """
        buildContext.logger.error(str(taskErr))

    def log_info(self, buildContext, msg):
        """Log an informational message using the logging facility """
        buildContext.logger.info('<' + self._TASK_NAME + '>: ' + msg)

    def is_valid_build_context(self, buildContext):
        """Returns True if the image under constructed is mounted; raises a fatal BuildError otherwise. """
        if not hasattr(buildContext, 'mountPoint') or not os.path.ismount(buildContext.mountPoint):
            raise BuildError(self._TASK_NAME, TaskError.ABORT_BUILD, 'not mounted:' + buildContext.mountPoint)
        return True
